@@ -17,8 +17,8 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2012 Aleksander Morgado <aleksander@lanedo.com>
  * Copyright (C) 2012 Google, Inc.
+ * Copyright (C) 2012-2015 Aleksander Morgado <aleksander@aleksander.es>
  */
 
 #ifndef _LIBQMI_GLIB_H_
@@ -65,6 +65,9 @@
 #include "qmi-enums-wda.h"
 #include "qmi-wda.h"
 
+#include "qmi-enums-voice.h"
+#include "qmi-voice.h"
+
 /* generated */
 #include "qmi-error-types.h"
 #include "qmi-enum-types.h"
