@@ -48,6 +48,9 @@ static gboolean operation_status;
 /* Main options */
 static gchar *device_str;
 static gboolean get_service_version_info_flag;
+static gboolean get_wwan_iface_flag;
+static gboolean get_expected_data_format_flag;
+static gchar *set_expected_data_format_str;
 static gchar *device_set_instance_id_str;
 static gboolean device_open_version_info_flag;
 static gboolean device_open_sync_flag;
@@ -64,6 +67,18 @@ static GOptionEntry main_entries[] = {
       "Specify device path",
       "[PATH]"
     },
+    { "get-wwan-iface", 'w', 0, G_OPTION_ARG_NONE, &get_wwan_iface_flag,
+      "Get the WWAN iface name associated with this control port",
+      NULL
+    },
+    { "get-expected-data-format", 'e', 0, G_OPTION_ARG_NONE, &get_expected_data_format_flag,
+      "Get the expected data format in the WWAN iface",
+      NULL
+    },
+    { "set-expected-data-format", 'E', 0, G_OPTION_ARG_STRING, &set_expected_data_format_str,
+      "Set the expected data format in the WWAN iface",
+      "[802-3|raw-ip]"
+    },
     { "get-service-version-info", 0, 0, G_OPTION_ARG_NONE, &get_service_version_info_flag,
       "Get service version info",
       NULL
@@ -208,7 +223,10 @@ generic_options_enabled (void)
         return !!n_actions;
 
     n_actions = (!!device_set_instance_id_str +
-                 get_service_version_info_flag);
+                 get_service_version_info_flag +
+                 get_wwan_iface_flag +
+                 get_expected_data_format_flag +
+                 !!set_expected_data_format_str);
 
     if (n_actions > 1) {
         g_printerr ("error: too many generic actions requested\n");
@@ -306,9 +324,15 @@ allocate_client_ready (QmiDevice *dev,
     case QMI_SERVICE_UIM:
         qmicli_uim_run (dev, QMI_CLIENT_UIM (client), cancellable);
         return;
+    case QMI_SERVICE_WMS:
+        qmicli_wms_run (dev, QMI_CLIENT_WMS (client), cancellable);
+        return;
     case QMI_SERVICE_WDA:
         qmicli_wda_run (dev, QMI_CLIENT_WDA (client), cancellable);
         return;
+    case QMI_SERVICE_VOICE:
+        qmicli_voice_run (dev, QMI_CLIENT_VOICE (client), cancellable);
+        return;
     default:
         g_assert_not_reached ();
     }
@@ -446,6 +470,89 @@ device_get_service_version_info (QmiDevice *dev)
                                          NULL);
 }
 
+static gboolean
+device_set_expected_data_format_cb (QmiDevice *dev)
+{
+    QmiDeviceExpectedDataFormat expected;
+    GError *error = NULL;
+
+    if (!qmicli_read_expected_data_format_from_string (set_expected_data_format_str, &expected) ||
+        expected == QMI_DEVICE_EXPECTED_DATA_FORMAT_UNKNOWN)
+        g_printerr ("error: invalid requested data format: %s", set_expected_data_format_str);
+    else if (!qmi_device_set_expected_data_format (dev, expected, &error)) {
+        g_printerr ("error: cannot set expected data format: %s\n", error->message);
+        g_error_free (error);
+    } else
+        g_print ("[%s] expected data format set to: %s\n",
+                 qmi_device_get_path_display (dev),
+                 qmi_device_expected_data_format_get_string (expected));
+
+    /* We're done now */
+    qmicli_async_operation_done (!error);
+
+    g_object_unref (dev);
+    return FALSE;
+}
+
+static void
+device_set_expected_data_format (QmiDevice *dev)
+{
+    g_debug ("Setting expected WWAN data format this control port...");
+    g_idle_add ((GSourceFunc) device_set_expected_data_format_cb, g_object_ref (dev));
+}
+
+static gboolean
+device_get_expected_data_format_cb (QmiDevice *dev)
+{
+    QmiDeviceExpectedDataFormat expected;
+    GError *error = NULL;
+
+    expected = qmi_device_get_expected_data_format (dev, &error);
+    if (expected == QMI_DEVICE_EXPECTED_DATA_FORMAT_UNKNOWN) {
+        g_printerr ("error: cannot get expected data format: %s\n", error->message);
+        g_error_free (error);
+    } else
+        g_print ("%s\n", qmi_device_expected_data_format_get_string (expected));
+
+    /* We're done now */
+    qmicli_async_operation_done (!error);
+
+    g_object_unref (dev);
+    return FALSE;
+}
+
+static void
+device_get_expected_data_format (QmiDevice *dev)
+{
+    g_debug ("Getting expected WWAN data format this control port...");
+    g_idle_add ((GSourceFunc) device_get_expected_data_format_cb, g_object_ref (dev));
+}
+
+static gboolean
+device_get_wwan_iface_cb (QmiDevice *dev)
+{
+    const gchar *wwan_iface;
+
+    wwan_iface = qmi_device_get_wwan_iface (dev);
+    if (!wwan_iface)
+        g_printerr ("error: cannot get WWAN interface name\n");
+    else
+        g_print ("%s\n", wwan_iface);
+
+    /* We're done now */
+    qmicli_async_operation_done (!!wwan_iface);
+
+    g_object_unref (dev);
+    return FALSE;
+}
+
+static void
+device_get_wwan_iface (QmiDevice *dev)
+{
+    g_debug ("Getting WWAN iface for this control port...");
+    g_idle_add ((GSourceFunc) device_get_wwan_iface_cb, g_object_ref (dev));
+}
+
 static void
 device_open_ready (QmiDevice *dev,
                    GAsyncResult *res)
@@ -465,6 +572,12 @@ device_open_ready (QmiDevice *dev,
         device_set_instance_id (dev);
     else if (get_service_version_info_flag)
         device_get_service_version_info (dev);
+    else if (get_wwan_iface_flag)
+        device_get_wwan_iface (dev);
+    else if (get_expected_data_format_flag)
+        device_get_expected_data_format (dev);
+    else if (set_expected_data_format_str)
+        device_set_expected_data_format (dev);
     else
         device_allocate_client (dev);
 }
@@ -546,12 +659,24 @@ parse_actions (void)
         actions_enabled++;
     }
 
+    /* WMS options? */
+    if (qmicli_wms_options_enabled ()) {
+        service = QMI_SERVICE_WMS;
+        actions_enabled++;
+    }
+
     /* WDA options? */
     if (qmicli_wda_options_enabled ()) {
         service = QMI_SERVICE_WDA;
         actions_enabled++;
     }
 
+    /* VOICE options? */
+    if (qmicli_voice_options_enabled ()) {
+        service = QMI_SERVICE_VOICE;
+        actions_enabled++;
+    }
+
     /* Cannot mix actions from different services */
     if (actions_enabled > 1) {
         g_printerr ("error: cannot execute multiple actions of different services\n");
@@ -579,25 +704,29 @@ int main (int argc, char **argv)
 
     /* Setup option context, process it and destroy it */
     context = g_option_context_new ("- Control QMI devices");
-	g_option_context_add_group (context,
-	                            qmicli_dms_get_option_group ());
-	g_option_context_add_group (context,
-	                            qmicli_nas_get_option_group ());
-	g_option_context_add_group (context,
-	                            qmicli_wds_get_option_group ());
-	g_option_context_add_group (context,
-	                            qmicli_pbm_get_option_group ());
-	g_option_context_add_group (context,
-	                            qmicli_uim_get_option_group ());
-	g_option_context_add_group (context,
-	                            qmicli_wda_get_option_group ());
+    g_option_context_add_group (context,
+                                qmicli_dms_get_option_group ());
+    g_option_context_add_group (context,
+                                qmicli_nas_get_option_group ());
+    g_option_context_add_group (context,
+                                qmicli_wds_get_option_group ());
+    g_option_context_add_group (context,
+                                qmicli_pbm_get_option_group ());
+    g_option_context_add_group (context,
+                                qmicli_uim_get_option_group ());
+    g_option_context_add_group (context,
+                                qmicli_wms_get_option_group ());
+    g_option_context_add_group (context,
+                                qmicli_wda_get_option_group ());
+    g_option_context_add_group (context,
+                                qmicli_voice_get_option_group ());
     g_option_context_add_main_entries (context, main_entries, NULL);
     if (!g_option_context_parse (context, &argc, &argv, &error)) {
         g_printerr ("error: %s\n",
                     error->message);
         exit (EXIT_FAILURE);
     }
-	g_option_context_free (context);
+    g_option_context_free (context);
 
     if (version_flag)
         print_version_and_exit ();
